
package com.nisum.offertransactionservice.controller;

import com.nisum.offertransactionservice.client.ClientService;
import com.nisum.offertransactionservice.dao.OfferLookupRepo;
import com.nisum.offertransactionservice.model.*;
import com.nisum.offertransactionservice.service.OfferCallingPEService;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.test.context.junit4.SpringRunner;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

import static org.mockito.Mockito.when;

@RunWith(SpringRunner.class)
@SpringBootTest
public class OfferControllerComponentTest {

    @Autowired
    private OfferTransactionController offerTransactionController;

    @InjectMocks
    private OfferCallingPEService offerCallingPEService = new OfferCallingPEService();

    @MockBean
    private OfferLookupRepo offerLookupRepo;

    @MockBean
    private ClientService clientService;

    @Before
    public void setUp() throws Exception {
        MockitoAnnotations.initMocks(this);
    }

    @Test
    public void test() throws InterruptedException {
        when(offerLookupRepo.findAll()).thenReturn(returnOffers());
        when(clientService.getPeResponseFlux(ArgumentMatchers.any())).thenReturn(getPeResponse());
        OfferTransactionResponse offerTransactionResponse = offerTransactionController.getOfferTransactionResponse(getOfferTransactionRequest());
        Assert.assertEquals(Objects.nonNull(offerTransactionResponse.getTransactionId()),true);
    }



    private PERequest getPEReq() {
        PERequest peRequest = new PERequest();
        peRequest.setHhId("123");
        List<Item> itemList = new ArrayList<>();
        Item item1 = new Item("Coke", "12", 6.05);
        Item item2 = new Item("Thumpsup", "11", 12.32);
        itemList.add(item1);
        itemList.add(item2);
        peRequest.setItemList(itemList);
        return peRequest;
    }

    private PEResponse getPeResponse() {
        PEResponse peResponse = new PEResponse();
        peResponse.setHhid("123");
        List<Item> itemList = new ArrayList<>();
        Item item1 = new Item("Coke", "12", 6.05);
        Item item2 = new Item("Thumpsup", "11", 12.32);
        itemList.add(item1);
        itemList.add(item2);
        peResponse.setDiscountedItemList(itemList);
        return peResponse;
    }

    public List<OfferLookup> returnOffers() {
        OfferLookup offerLookup1 = new OfferLookup("1", "12", "234", "02", "AND", "BOGO");
        OfferLookup offerLookup2 = new OfferLookup("2", "13", "234", "02", "OR", "BXGY");
        List<OfferLookup> offerLookups = new ArrayList<>();
        offerLookups.add(offerLookup1);
        offerLookups.add(offerLookup2);
        return offerLookups.stream().collect(Collectors.toList());
    }

    private OfferTransactionRequest getOfferTransactionRequest() {
        OfferTransactionRequest offerTransactionRequest = new OfferTransactionRequest();
        offerTransactionRequest.setHhId("123");
        List<Item> itemList = new ArrayList<>();
        Item item1 = new Item("Coke", "12", 13.05);
        Item item2 = new Item("Thumpsup", "11", 14.05);
        itemList.add(item1);
        itemList.add(item2);
        offerTransactionRequest.setItemList(itemList);
        return offerTransactionRequest;
    }

}

